/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.serviceregistry;

import java.util.List;

import javax.xml.registry.infomodel.ClassificationScheme;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;
import javax.xml.registry.infomodel.ServiceBinding;

public interface IServiceRegistry {

	// Factory Method 
	public Organization newOrganization(String organizationName, String organizationDescription) throws ServiceRegistryException;
	public Service newService(String serviceName, String serviceDescription)throws ServiceRegistryException;
	public ServiceBinding newServiceBinding(String serviceName,String serviceBindingLocation) throws ServiceRegistryException;
	public ClassificationScheme newClassificationScheme(String cSchemeName,String cSchemeDescription) throws ServiceRegistryException;
	public Organization[] getOrganizationsByName(String organizationName) throws ServiceRegistryException;
	
	// Queries
	public Organization getOrganizationByName(String organizatioName) throws ServiceRegistryException;
	public Service getServiceByOrganizationAndServiceName(Organization org, String serviceName) throws ServiceRegistryException;
	public Service getServiceByOrganizationNameAndServiceName(String organizationName, String serviceName) throws ServiceRegistryException;
	public ClassificationScheme getClassificationSchemeByName(String cSchemeName) throws ServiceRegistryException;
	
	// Persistence Support for Organizations
	public Organization publishOrganization(Organization organization) throws ServiceRegistryException;
	public void deleteOrganization(Organization organization) throws ServiceRegistryException;
	
	// Persistence Support for Services
	public Service publishService(Organization organization, Service service, List<ServiceBinding> bindings) throws ServiceRegistryException;
	public void deleteService(Organization organization, Service service) throws ServiceRegistryException;
	

	// Persistence Support for Classification Scheme or Taxonomy
	public ClassificationScheme publishClassificationScheme(ClassificationScheme classificationScheme) throws ServiceRegistryException;
	public void deleteClassificationScheme(ClassificationScheme classificationScheme) throws ServiceRegistryException;
	
	
	// Add and remove values to taxonomy
	public void addValueToClassificationScheme(ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException;
	public void removeValueFromClassificationScheme(ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException; 
	public void removeAllValuesFromClassificationScheme(ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException; 
	
	
	
	public List<Service> getServicesForOrganizations(Organization organization) throws ServiceRegistryException;
	public List<Service> getServicesClassifiedByTaxonomy(ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException;
	public List<Service> getServicesClassifiedByTaxonomy(String classificationSchemeName, String classificationValue) throws ServiceRegistryException;
	
	public void classifyServiceByTaxonomy(Service service, ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException;
	
	public void removeAllClassificationForService(Service service) throws ServiceRegistryException;
	public void removeAllClassificationForServiceByClassification(ClassificationScheme cScheme,Service service) throws ServiceRegistryException;

	public List<Organization> getAllOrganizations() throws ServiceRegistryException;;
	public List<ClassificationScheme> getAllClassificationScheme() throws ServiceRegistryException;;
	public List<String> getClassificationSchemeValues(String classificationSchemeName) throws ServiceRegistryException;
	
	public ServiceBinding setWsdlLocation(String serviceName, ServiceBinding binding, String wsdlLocation) throws ServiceRegistryException;
	
	public List<String> getAllClassificationsForService(Organization org, Service ser) throws ServiceRegistryException;

	public void initRegistry();
	public void finalizeRegistry();
}
