/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.serviceregistry;

import java.io.InputStream;
import java.net.PasswordAuthentication;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.sql.DataSource;
import javax.wsdl.xml.WSDLLocator;
import javax.xml.registry.BulkResponse;
import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.BusinessQueryManager;
import javax.xml.registry.Connection;
import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.FindQualifier;
import javax.xml.registry.LifeCycleManager;
import javax.xml.registry.RegistryService;
import javax.xml.registry.infomodel.Classification;
import javax.xml.registry.infomodel.ClassificationScheme;
import javax.xml.registry.infomodel.Concept;
import javax.xml.registry.infomodel.ExternalLink;
import javax.xml.registry.infomodel.InternationalString;
import javax.xml.registry.infomodel.Key;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;
import javax.xml.registry.infomodel.ServiceBinding;
import javax.xml.registry.infomodel.SpecificationLink;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.dbutils.DbUtils;
import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.handlers.MapListHandler;
import org.apache.juddi.uuidgen.DefaultUUIDGen;
import org.apache.ws.scout.registry.infomodel.ClassificationSchemeImpl;
import org.apache.ws.scout.registry.infomodel.InternationalStringImpl;
import org.apache.ws.scout.registry.infomodel.KeyImpl;

public class JAXRServiceRegistryImpl implements IServiceRegistry {
	
	private Properties properties = null;
	private ConnectionFactory connectionFactory = null;
	private DataSource uddiDataSource = null;
	private Locale systemDefaultLocale = null;
	
	public JAXRServiceRegistryImpl() throws  ServiceRegistryException {
		
		InputStream is = null;
		try {
			is = Thread.currentThread().getContextClassLoader().getResourceAsStream("ServiceRegistry.properties");
			this.properties = new Properties();
			this.properties.load(is);

			String connectionFactoryClass = (String) properties
			.get("javax.xml.registry.ConnectionFactoryClass");
			if (connectionFactoryClass == null)
				connectionFactoryClass = "org.apache.ws.scout.registry.ConnectionFactoryImpl";
			System.setProperty("javax.xml.registry.ConnectionFactoryClass",
					connectionFactoryClass);
			this.connectionFactory = ConnectionFactory.newInstance();
			this.connectionFactory.setProperties(properties);

	 
			this.uddiDataSource  = new BasicDataSource();
			((BasicDataSource)this.uddiDataSource).setDriverClassName((String)this.properties.get("metadb.db.driver"));
			((BasicDataSource)this.uddiDataSource).setUsername((String)this.properties.get("metadb.db.user"));
			((BasicDataSource)this.uddiDataSource).setPassword((String)this.properties.get("metadb.db.password"));
			((BasicDataSource)this.uddiDataSource).setUrl((String)this.properties.get("metadb.db.url"));
			
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		} finally {
			try {
				is.close();
			} catch (Exception ex) {

			}
		}
	}
	
	
	
	public JAXRServiceRegistryImpl(Properties prop) throws  ServiceRegistryException {
		try {
		
			this.properties = prop;
		
			String connectionFactoryClass = (String) properties
			.get("javax.xml.registry.ConnectionFactoryClass");
			if (connectionFactoryClass == null)
				connectionFactoryClass = "org.apache.ws.scout.registry.ConnectionFactoryImpl";
			System.setProperty("javax.xml.registry.ConnectionFactoryClass",
					connectionFactoryClass);
			this.connectionFactory = ConnectionFactory.newInstance();
			this.connectionFactory.setProperties(properties);

	 
			this.uddiDataSource  = new BasicDataSource();
			((BasicDataSource)this.uddiDataSource).setDriverClassName((String)this.properties.get("metadb.db.driver"));
			((BasicDataSource)this.uddiDataSource).setUsername((String)this.properties.get("metadb.db.user"));
			((BasicDataSource)this.uddiDataSource).setPassword((String)this.properties.get("metadb.db.password"));
			((BasicDataSource)this.uddiDataSource).setUrl((String)this.properties.get("metadb.db.url"));
			
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		} finally {
			
		}
	}
	
	// Connection Handling
	private Connection getJAXRConnection() throws  ServiceRegistryException {
        try{
        	Connection connection = connectionFactory.createConnection();
        	String registryUser = properties.getProperty("registry.username");
        	String registryPassword = properties.getProperty("registry.password");
        	PasswordAuthentication pa = new PasswordAuthentication(registryUser, registryPassword.toCharArray());
        	Set credentials = new HashSet();
        	credentials.add(pa);
        	connection.setCredentials(credentials);
        	return connection;
        }catch(Exception e){
        	throw new ServiceRegistryException(e);
        }
	}
	
	private static void closeJAXRConnection(Connection aJAXRConnection)  {
		try{
			if (aJAXRConnection !=null && !aJAXRConnection.isClosed())
				aJAXRConnection.close();
		}catch (Exception e) {
			e.printStackTrace();
		}
	}


	public Organization[] getOrganizationsByName(String organizationName)
	throws ServiceRegistryException {
Connection registryConnection = null;

try {
	registryConnection = getJAXRConnection();
	RegistryService registryService = registryConnection
			.getRegistryService();
	BusinessQueryManager bqm = registryService
			.getBusinessQueryManager();

	Collection<String> findQualifiers = new ArrayList<String>();
	findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
	Collection<String> namePatterns = new ArrayList<String>();
	namePatterns.add(organizationName);

	BulkResponse response = bqm.findOrganizations(findQualifiers,
			namePatterns, null, null, null, null);
	Collection orgs = response.getCollection();
	System.out.println(" Orgs.size["+orgs.size()+"]");
	Organization[] ret = new Organization[orgs.size()];
	int i=0;
	for (Iterator it = orgs.iterator(); it.hasNext();){
		ret[i] = (Organization)it.next();
		i++;
	}
	return ret;
} catch (Exception e) {
	e.printStackTrace();
	throw new ServiceRegistryException(e);
} finally {
	closeJAXRConnection(registryConnection);
}
}
	

	public Organization getOrganizationByName(String organizationName)
			throws ServiceRegistryException {
		Connection registryConnection = null;
		
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection
					.getRegistryService();
			BusinessQueryManager bqm = registryService
					.getBusinessQueryManager();

			Collection<String> findQualifiers = new ArrayList<String>();
			findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
			Collection<String> namePatterns = new ArrayList<String>();
			namePatterns.add(organizationName);

			BulkResponse response = bqm.findOrganizations(findQualifiers,
					namePatterns, null, null, null, null);
			Collection orgs = response.getCollection();
			System.out.println(" Orgs.size["+orgs.size()+"]");
			if (orgs.size() > 1) {
				throw new ServiceRegistryException(
						"Found more than one result for this organization name");
			} else {
				Organization org = (Organization) orgs.toArray()[0];
				return org;
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			throw new ServiceRegistryException(e);
		} finally {
			closeJAXRConnection(registryConnection);
		}
	}


	public void deleteOrganization(Organization organization) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection
					.getRegistryService();
			BusinessLifeCycleManager lifeCycleManager = registryService
					.getBusinessLifeCycleManager();
			String id = organization.getKey().getId();
			System.out.println("Deleting organization with id " + id);

			Collection<Key> keys = new ArrayList<Key>();
			keys.add(organization.getKey());

			BulkResponse response = lifeCycleManager.deleteOrganizations(keys);
			Collection exceptions = response.getExceptions();

			if (exceptions == null) {
				System.out.println("Organization deleted");

				Collection retKeys = response.getCollection();
				for (Object k : retKeys) {
					Key orgKey = (Key) k;
					id = orgKey.getId();
					System.out.println("Organization key was " + id);
				}
			} else {
				for (Object e : exceptions) {
					Exception exception = (Exception) e;
					System.err.println("Exception on delete: "
							+ exception.toString());
				}
			}
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}



	public void deleteService(Organization organization, Service service) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();
			
			Organization org = (Organization)bqm.getRegistryObject(organization.getKey().getId(), LifeCycleManager.ORGANIZATION);
			Service ser = (Service)bqm.getRegistryObject(service.getKey().getId(), LifeCycleManager.SERVICE);
			ser.setProvidingOrganization(org);
			removeAllClassificationForService(ser);
			org.removeService(ser);
			publishOrganization(org);
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
		
	}

	public List<Service> getServicesClassifiedByTaxonomy(ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException  {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			List<Service> returnedServices = new ArrayList<Service>();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();
			String sql = "SELECT SC.SERVICE_UDDI_NAME, SC.SERVICE_UDDI_ORG FROM SERVICE_CLASSIFICATION SC WHERE SC.CSCHEME_ID='"+classificationScheme.getKey().getId()+"' AND SC.CSCHEME_VALUE='"+classificationValue+"'";
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			List<Map> mapList =(List) runner.query(sql, new MapListHandler());
			String serviceuddiName = null;
			String serviceuddiOrg = null;
			for (Map m : mapList){
				serviceuddiName = (String)m.get("SERVICE_UDDI_NAME");
				serviceuddiOrg = (String)m.get("SERVICE_UDDI_ORG");
				
				//returnedServices.add((Service)bqm.getRegistryObject(serviceuddiId,LifeCycleManager.SERVICE));
				returnedServices.add(getServiceByOrganizationNameAndServiceName(serviceuddiOrg, serviceuddiName));
			}
			
			return returnedServices;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}



	public List<Service> getServicesForOrganizations(Organization organization) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();
			Organization org = (Organization)bqm.getRegistryObject(organization.getKey().getId(), LifeCycleManager.ORGANIZATION);
			
			List<Service> servicesList = new ArrayList<Service>();
		
			Collection services = org.getServices();
			Iterator svcIter = services.iterator();
			while (svcIter.hasNext()) {
				Service svc = (Service) svcIter.next();
				servicesList.add(svc);
			}
		return servicesList;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}



	public Organization publishOrganization(Organization organization) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
        
			Collection orgs = new ArrayList();
			orgs.add(organization);

			//Save the Organization in the UDDI registry.
			BulkResponse bulkResponse = blcm.saveOrganizations(orgs);

			//Obtain the Organization's Key (the UDDI businessEntity's businessKey) from the response.
			if (bulkResponse.getExceptions() == null)
			{
				//1 Organization was saved, so 1 key will be returned in the response collection
				Collection responses = bulkResponse.getCollection();
				Key organizationKey = (Key)responses.iterator().next();
				System.out.println("\nOrganization Key = " + organizationKey.getId() + "Published");
				organization.setKey(organizationKey);
				return organization;
			}else{
				return null;
			}
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}

	

	public Service publishService(Organization organization, Service service, List<ServiceBinding> serviceBindings) throws ServiceRegistryException {
		
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			//Create services and service
			Collection<Service> services = new ArrayList<Service>();
			//	 Add service bindings to service
			if (serviceBindings != null)
				service.addServiceBindings(serviceBindings);
			//   Add service to services, then add services to organization
			service.setProvidingOrganization(organization);
			services.add(service);
			organization.addServices(services);
			publishOrganization(organization);
			return service;
	} catch (Exception e) {
		throw new ServiceRegistryException(e);
	}finally {
		closeJAXRConnection(registryConnection);
	}
	}

	public ServiceBinding setWsdlLocation(String serviceName, ServiceBinding binding, String wsdlLocation) throws ServiceRegistryException{
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();
		
			if (wsdlLocation != null){
				Concept specConcept =
					blcm.createConcept(null, blcm.createInternationalString("Concept for service "+ serviceName), "");
				InternationalString s = blcm.createInternationalString("Concept for service "+ serviceName);
				specConcept.setDescription(s);
				ExternalLink wsdlLink =
					blcm.createExternalLink(wsdlLocation,serviceName + "WSDL");
				specConcept.addExternalLink(wsdlLink);
    		
				// Classifying the concept as WSDL
				// String uuid_types = "uuid:c1acf26d-9672-4404-9d70-39b756e62ab4";
				String schemeName = "uddi-org:types";
				ClassificationScheme uddiOrgTypes =
					(ClassificationScheme) bqm.findClassificationSchemeByName(null, schemeName);
    			
				Classification wsdlSpecClassification =
					blcm.createClassification(uddiOrgTypes,
							"wsdlSpec", "wsdlSpec");
				specConcept.addClassification(wsdlSpecClassification);
    		
    		
				Collection<Concept> concepts = new ArrayList<Concept>();
				concepts.add(specConcept);
    			BulkResponse concResponse = blcm.saveConcepts(concepts);
    		
    			String conceptKeyId = null;
    			Collection concExceptions = concResponse.getExceptions();
    			Key concKey = null;
    			if (concExceptions == null) {
    				System.out.println("WSDL Specification Concept saved");
    				Collection keys = concResponse.getCollection();
    				Iterator<Key> keyIter = keys.iterator();
    				if (keyIter.hasNext()) {
    					concKey = (Key) keyIter.next();
    					conceptKeyId = concKey.getId();
    					System.out.println("Concept key is " + conceptKeyId);
    				}
    			}
    		
    		
    			specConcept = (Concept) bqm.getRegistryObject(conceptKeyId,
    					LifeCycleManager.CONCEPT);
    			
    			SpecificationLink specLink = blcm.createSpecificationLink();
    			specLink.setSpecificationObject(specConcept);
    			binding.addSpecificationLink(specLink);
			}
			return binding;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}

	public Organization newOrganization(String organizationName, String organizationDescription) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
	        BusinessLifeCycleManager lifeCycleManager = registryService.getBusinessLifeCycleManager();
	        Organization organization = lifeCycleManager.createOrganization(organizationName);
	        organization.setDescription(lifeCycleManager.createInternationalString());
	        return organization;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}



	public Service newService(String serviceName, String serviceDescription) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			InternationalString iServiceName = blcm.createInternationalString(serviceName);
			Service service = blcm.createService(iServiceName);
			InternationalString iServiceDescription = blcm.createInternationalString(serviceDescription);
			service.setDescription(iServiceDescription);
			return service;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}



	public ServiceBinding newServiceBinding(String serviceName, String serviceBindingLocation) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();	
			ServiceBinding binding = blcm.createServiceBinding();
			InternationalString iServiceBindingDescritpion = blcm.createInternationalString("Binding For " + serviceName );
			binding.setDescription(iServiceBindingDescritpion);
			//	 allow us to publish a fictitious URI without an error
			binding.setValidateURI(false);
			binding.setAccessURI(serviceBindingLocation);
			return binding;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}


	public void addValueToClassificationScheme(ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException {
		
		try {

			String sql = "INSERT INTO CSCHEME_VALUES VALUES('"+classificationScheme.getKey().getId()+"', '"+classificationValue +"')";
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			runner.update(sql);
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			
		}
		
	}


	public void deleteClassificationScheme(ClassificationScheme classificationScheme) throws ServiceRegistryException {
		
		try {
			
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			String sql = "DELETE FROM SERVICE_CLASSIFICATION WHERE CSCHEME_ID='"+classificationScheme.getKey().getId()+"'";
			runner.update(sql);
			
			// Delete taxonomy values
			sql = "DELETE FROM CSCHEME_VALUES WHERE CSCHEME_ID ='"+classificationScheme.getKey().getId()+"'";
			runner.update(sql);
			
			// Delete taxonomy
			sql = "DELETE FROM CSCHEME WHERE CSCHEME_ID ='"+classificationScheme.getKey().getId()+"'";
			runner.update(sql);
			
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			
		}
		
	}


	public ClassificationScheme newClassificationScheme(String cSchemeName, String cSchemeDescription) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			String cschemeUUID = new DefaultUUIDGen().uuidgen();
			
			ClassificationScheme cScheme =  new ClassificationSchemeImpl(null);
			cScheme.setKey(new KeyImpl(cschemeUUID));
			cScheme.setName(blcm.createInternationalString(cSchemeName));
			cScheme.setDescription(blcm.createInternationalString(cSchemeDescription));
			return cScheme;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}


	public ClassificationScheme publishClassificationScheme(ClassificationScheme classificationScheme) throws ServiceRegistryException {
		
		try {
			
			String sql = "INSERT INTO CSCHEME VALUES('"+classificationScheme.getKey().getId()+"', '"+classificationScheme.getName().getValue() +"', '"+classificationScheme.getDescription().getValue() +"')";
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			runner.update(sql);
			return classificationScheme;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			
		}
			
	}


	public void removeAllValuesFromClassificationScheme(ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException {
		
		try {
			
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			
			// Remove Service Classification based on this taxonomy and on this value
			String sql = "DELETE FROM SERVICE_CLASSIFICATION WHERE CSCHEME_ID='"+classificationScheme.getKey().getId()+"' AND CSCHEME_VALUE='"+classificationValue+"'";
			runner.update(sql);
			
			// Remove Taxonomy value
			sql = "DELETE FROM CSCHEME_VALUES WHERE CSCHEME_ID ='"+classificationScheme.getKey().getId()+"'";
			runner.update(sql);
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			
		}
	}


	public void removeValueFromClassificationScheme(ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException {
		
		try {
			
			String sql = "DELETE FROM CSCHEME_VALUES WHERE CSCHEME_ID ='"+classificationScheme.getKey().getId()+"' AND C.CSCHEME_VALUE='"+classificationValue+"'";
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			runner.update(sql);
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			
		}
	}


	public ClassificationScheme getClassificationSchemeByName(String cSchemeName) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			
			
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			
			String sql = "SELECT * FROM CSCHEME WHERE CSCHEME_NAME='"+cSchemeName+"'";
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			List<Map> mapList =(List) runner.query(sql, new MapListHandler());
			if (mapList.size() > 0){
				Map m = mapList.get(0);
			
				ClassificationScheme cScheme =  new ClassificationSchemeImpl(null);
				cScheme.setKey(new KeyImpl((String)m.get("CSCHEME_ID")));
				cScheme.setName(blcm.createInternationalString((String)m.get("CSCHEME_NAME")));
				cScheme.setDescription(blcm.createInternationalString((String)m.get("CSCHEME_DESCRIPTION")));
				return cScheme;
			}else{
				return null;
			}
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
		
		
	}

	public Service getServiceByOrganizationNameAndServiceName(String organizationName, String serviceName) throws ServiceRegistryException {
		Organization org = getOrganizationByName(organizationName);
		return getServiceByOrganizationAndServiceName(org, serviceName);
	}
	
	public Service getServiceByOrganizationAndServiceName(Organization org, String serviceName) throws ServiceRegistryException {	
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			BusinessQueryManager bqm = registryService.getBusinessQueryManager();
			

			Collection<String> findQualifiers = new ArrayList<String>();
			findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
			Collection<String> namePatterns = new ArrayList<String>();
			namePatterns.add(serviceName);

			bqm.findOrganizations(findQualifiers, namePatterns, null,
			null, null, null);
			
			BulkResponse response = bqm.findServices(org.getKey(), findQualifiers, namePatterns,null,null);
			Collection services = response.getCollection();			
			if (services.size() > 1){
				throw new ServiceRegistryException(" Founded more that one service named ["+serviceName+"] in Organization["+org.getName()+"]");
			}else{
				Service aService = (Service)services.iterator().next();
				
				// SCOUT BUGS
				aService.setProvidingOrganization(org);
				return aService;
			}
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		} finally {
			closeJAXRConnection(registryConnection);
		}
			
	}

	public void classifyServiceByTaxonomy(Service service, ClassificationScheme classificationScheme, String classificationValue) throws ServiceRegistryException{
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			String sql = "INSERT INTO SERVICE_CLASSIFICATION VALUES('"+classificationScheme.getKey().getId()+"', '"+classificationValue+"', '"+service.getProvidingOrganization().getName().getValue()+"','"+service.getName().getValue()+"')";
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			runner.update(sql);
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		} finally {
			closeJAXRConnection(registryConnection);
		}
	}

	public void removeAllClassificationForService(Service service) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			String sql = "DELETE FROM SERVICE_CLASSIFICATION WHERE SERVICE_UDDI_ORG='"+service.getProvidingOrganization().getName().getValue()+"' AND SERVICE_UDDI_NAME='"+service.getName().getValue()+"'";
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			runner.update(sql);
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		} finally {
			closeJAXRConnection(registryConnection);
		}
		
	}

	public void removeAllClassificationForServiceByClassification(ClassificationScheme cScheme, Service service) throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			registryConnection = getJAXRConnection();
			String sql = "DELETE FROM SERVICE_CLASSIFICATION WHERE SERVICE_UDDI_ORG='"+service.getProvidingOrganization().getName().getValue()+"' AND SERVICE_UDDI_NAME='"+service.getName().getValue()+"' AND CSCHEME_ID='"+cScheme.getKey().getId()+"'";
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			runner.update(sql);
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		} finally {
			closeJAXRConnection(registryConnection);
		}
		
	}

	public List<ClassificationScheme> getAllClassificationScheme() throws ServiceRegistryException {
		Connection registryConnection = null;
		try {
			
			List<ClassificationScheme> returnedCSchemes = new ArrayList<ClassificationScheme>();
			registryConnection = getJAXRConnection();
			RegistryService registryService = registryConnection.getRegistryService();
			BusinessLifeCycleManager blcm = registryService.getBusinessLifeCycleManager();
			
			String sql = "SELECT * FROM CSCHEME";
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			List<Map> mapList =(List) runner.query(sql, new MapListHandler());
			for (Map m : mapList){
				ClassificationScheme cScheme =  new ClassificationSchemeImpl(null);
				cScheme.setKey(new KeyImpl((String)m.get("CSCHEME_ID")));
				cScheme.setName(blcm.createInternationalString((String)m.get("CSCHEME_NAME")));
				cScheme.setDescription(blcm.createInternationalString((String)m.get("CSCHEME_DESCRIPTION")));
				returnedCSchemes.add(cScheme);
			}
			return returnedCSchemes;
		} catch (Exception e) {
			throw new ServiceRegistryException(e);
		}finally {
			closeJAXRConnection(registryConnection);
		}
	}

	public List<Organization> getAllOrganizations() throws ServiceRegistryException {
			Connection registryConnection = null;
			try {
				registryConnection = getJAXRConnection();
				RegistryService registryService = registryConnection
						.getRegistryService();
				BusinessQueryManager bqm = registryService
						.getBusinessQueryManager();

				Collection<String> findQualifiers = new ArrayList<String>();
				findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
				Collection<String> namePatterns = new ArrayList<String>();
				namePatterns.add("%");
			
				BulkResponse response = bqm.findOrganizations(findQualifiers,
						namePatterns, null, null, null, null);
				Collection<Organization> orgs = response.getCollection();
				List<Organization> result = new ArrayList<Organization>();
				for (Organization org : orgs){
					result.add(org);
				}
				return result;
			} catch (Exception e) {
				throw new ServiceRegistryException(e);
			} finally {
				closeJAXRConnection(registryConnection);
			}
	}

	public List<String> getClassificationSchemeValues(String classificationSchemeName) throws ServiceRegistryException{
		
		try {
			
				String sql = "SELECT CV.CSCHEME_VALUE FROM CSCHEME_VALUES CV, CSCHEME C " +
							 "WHERE C.CSCHEME_NAME = '"+classificationSchemeName+"'" + 
							 "AND CV.CSCHEME_ID = C.CSCHEME_ID";
				QueryRunner runner = new QueryRunner(this.uddiDataSource);
				List<Map> mapList =(List) runner.query(sql, new MapListHandler());
				List<String> cSchemeValues = new ArrayList<String>();
				for (Map m : mapList){
					cSchemeValues.add((String)m.get("CSCHEME_VALUE"));
				}
				return cSchemeValues;
			} catch (Exception e) {
				e.printStackTrace();
				throw new ServiceRegistryException(e);
			} finally {
				
			}
		}

	public List<Service> getServicesClassifiedByTaxonomy(String classificationSchemeName, String classificationValue) throws ServiceRegistryException {
	
			ClassificationScheme cScheme = getClassificationSchemeByName(classificationSchemeName);
			return getServicesClassifiedByTaxonomy(cScheme, classificationValue);
		
	}
	
	
	public List<String> getAllClassificationsForService(Organization org, Service ser) throws ServiceRegistryException{
		
		
		try {
			
			String sql = "SELECT CS.CSCHEME_NAME, SC.CSCHEME_VALUE " +
						 "FROM CSCHEME CS, " +
						 "SERVICE_CLASSIFICATION SC " +
						 "WHERE  SC.CSCHEME_ID=CS.CSCHEME_ID "+ 
						 "AND SC.SERVICE_UDDI_ORG='"+org.getName().getValue()+"' AND SC.SERVICE_UDDI_NAME='"+ser.getName().getValue()+"'";
			QueryRunner runner = new QueryRunner(this.uddiDataSource);
			List<Map> mapList =(List) runner.query(sql, new MapListHandler());
			List<String> returns = new ArrayList<String>();
			String str = null;
			for (Map m : mapList){
				str = new String();
				str += m.get("CSCHEME_NAME");
				str += ";";
				str += m.get("CSCHEME_VALUE");
				returns.add(str);
			}
			return returns;
		} catch (Exception e) {
			e.printStackTrace();
			throw new ServiceRegistryException(e);
		} finally {
			
		}
	}



	public void finalizeRegistry() {
		Locale.setDefault(this.systemDefaultLocale);
		
	}



	public void initRegistry() {
		this.systemDefaultLocale = Locale.getDefault();
		Locale.setDefault(Locale.US);
		
	}


	
	
	
}
